/**
This work-around solve two problems:
  1. validate user code - UM10375, chap. 19.7.
  2. correct write file to Usb Mass Storage device
    emulated by LPC1343. Description in AN10986.
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#define BUFLEN 0x1000

typedef unsigned int uint32_t;

enum retCodes {
  LPC_OK = 0,
  LPC_ENOMEM,
  LPC_NO_INPUT,
  LPC_NO_OUTPUT
};
void printUsage (void) {
  printf ("This software upload newfirmware.bin to LPC1343 device using USB mass storage\n"
    "Usage:\n"
    "\tlpcUsbCp newfirmware.bin [/path/to/mounted/usb/disk/firmware.bin]\n");
}
void validateUserCode (char* ptr) {
  uint32_t* vectors  = (uint32_t*) ptr;
  uint32_t  cnt, sum = 0;
  for (cnt=0; cnt<7; cnt++) {
    sum += vectors [cnt];
  }
  sum = ~sum;
  sum++;
  printf ("User code checksum is 0x%08X (0x%08X)\n", sum, vectors[cnt]);
  vectors[cnt] = sum;
}
/// main ()
int main (int argc, char *argv[]) {
  // do edit correct path to firmware.bin on mounted usb device or use command line parameter
  char* outFileName = "/media/CRP DISABLD/firmware.bin";
  //char* outFileName = "firmware.bin";
  char*  inFileName;
  FILE  *inFile, *outFile;
  char*  buffer;
  int    blkCount;
  size_t nReaden, nWriten;
  enum retCodes retCode = LPC_OK;

  if (sizeof (uint32_t) != 4) {
    fprintf (stderr, "Platform error !!!\n");
    return retCode;
  }
  switch (argc) {
    case 3:  outFileName = argv[2];
    case 2:   inFileName = argv[1];
      break;
    default: printUsage ();
      return retCode;
  }
  for (;;) {  // one cycle only - define output point - prevent usage goto
    buffer = malloc (BUFLEN);
    if (!buffer) {
      fprintf (stderr, "Not memory space\n");
      retCode = LPC_ENOMEM;
      break;
    }
    inFile = fopen (inFileName, "r");
    if (!inFile) {
      fprintf (stderr, "Cannot open  input file \"%s\", err: \"%s\"\n",  inFileName, strerror (errno));
      retCode = LPC_NO_INPUT;
      break;
    }
    outFile = fopen (outFileName, "r+");
    if (!inFile) {
      fprintf (stderr, "Cannot open output file \"%s\", err: \"%s\"\n", outFileName, strerror (errno));
      retCode = LPC_NO_OUTPUT;
      break;
    }
    for (blkCount = 0; ; blkCount++) {
      nReaden = fread (buffer, 1, BUFLEN, inFile);
      if (!nReaden) {
        fprintf (stderr, "Cannot read input file \"%s\"\n", inFileName);
        break;
      }
      if (!blkCount) validateUserCode (buffer);
      nWriten = fwrite (buffer, 1, nReaden, outFile);
      if (nWriten != nReaden) {
        fprintf (stderr, "Cannot write to output file \"%s\"\n", outFileName);
        break;
      }
      printf ("%d\tbytes writen\n", (int) nWriten);
      if (nReaden < BUFLEN) {
        if (blkCount > 8) fprintf (stderr, "Input file \"%s\" too long\n", inFileName);
        break;
      }
    }
    break;
  }                     // output point single cycle
  switch (retCode) {    // prevent goto
    case LPC_OK:
      printf ("End of load OK, blk=%d\n", blkCount);
      fclose (outFile);
    case LPC_NO_OUTPUT:
      fclose (inFile);
    case LPC_NO_INPUT:
      free (buffer);
    case LPC_ENOMEM:
    default:
      break;
  }
  return -retCode;
}
